package cn.ibizlab.core.lite.extensions.model;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.experimental.Accessors;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.LinkedHashSet;

@Getter
@Setter
@NoArgsConstructor
@Accessors(chain = true)
public class DataModel {

    @JsonIgnore
    @JSONField(serialize = false)
    private DataModel parentDataModel;

    @JSONField(ordinal = 1)
    private String dataModelName;
    public String getDataModelName()
    {
        if(StringUtils.isEmpty(dataModelName))
        {
            Property property = getFactPorperty();
            if(property!=null)
                return property.getPropertyName();
        }
        return dataModelName;
    }

    @JSONField(ordinal = 2)
    private LinkedHashSet<Property> objectProperties = new LinkedHashSet<>();
    public void setObjectProperties(LinkedHashSet<Property> objectProperties) {
        if(objectProperties!=null) {
            objectProperties.forEach(objectPorperty -> objectPorperty.setOwnerDataModel(this));
            this.objectProperties = objectProperties;
        }
    }

    @JSONField(ordinal = 3)
    private LinkedHashSet<DataModel> nestedDataModels = new LinkedHashSet<>();
    public void setNestedDataModels(LinkedHashSet<DataModel> nestedDataModels) {
        if(nestedDataModels!=null) {
            nestedDataModels.forEach(nestedDataModel -> nestedDataModel.setParentDataModel(this));
            this.nestedDataModels = nestedDataModels;
        }
    }

    @JSONField(ordinal = 4)
    private LinkedHashSet<LayerMapping> layerMappings = new LinkedHashSet<>();
    public void setLayerMappings(LinkedHashSet<LayerMapping> layerMappings)
    {
        if(layerMappings!=null) {
            layerMappings.forEach(layerMapping -> layerMapping.setSelfDataModel(this));
            this.layerMappings = layerMappings;
        }
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public Property getFactPorperty()
    {
        if(!ObjectUtils.isEmpty(this.getObjectProperties()))
        {
            return (Property)this.getObjectProperties().toArray()[0];
        }
        return null;
    }

    public Property getObjectProperty(String propertyName)
    {
        if(this.getObjectProperties()!=null&&(!StringUtils.isEmpty(propertyName)))
        {
            for (Property property:this.getObjectProperties())
                if(propertyName.equalsIgnoreCase(property.getPropertyName()))
                    return property;
        }
        return null;
    }

    public DataModel getNestedDataModel(String dataModelName)
    {
        if(this.getNestedDataModels()!=null&&(!StringUtils.isEmpty(dataModelName)))
        {
            for (DataModel dataModel:this.getNestedDataModels())
                if(dataModelName.equalsIgnoreCase(dataModel.getDataModelName()))
                    return dataModel;
        }
        return null;
    }

    @Override
    public String toString() {
        return "DataModel{" +
                "dataModelName='" + getDataModelName() + '\'' +
                ", objectProperties=" + getObjectProperties() +
                ", nestedDataModels=" + getNestedDataModels() +
                ", layerMappings=" + getLayerMappings() +
                "}\r\n";
    }
}
